package cn.com.qmth.stmms.admin.thread;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.ScoreRate;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.ScoreRateService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.utils.ScoreCalculateUtil;
import cn.com.qmth.stmms.biz.utils.ScoreInfo;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.common.enums.ExamStatus;
import cn.com.qmth.stmms.common.enums.LibraryStatus;

public class ScoreCalculateThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(ScoreCalculateThread.class);

    private ExamStudentService studentService;

    private ExamService examService;

    private ExamQuestionService questionService;
    
    private ExamSubjectService subjectService;
    
    private ScoreRateService scoreRateService;
    
    private MarkLibraryService libraryService;
    
    private Exam exam;

    private Map<String, List<ExamQuestion>> objectiveMap;
    
    private Map<String, List<ExamQuestion>> subjectiveMap;
    
    private Map<String,Set<String>> campusNameMap;
    
    private Map<String,Double> scoreRateMap;

    public ScoreCalculateThread(Exam exam, ExamService examService, ExamStudentService studentService,
            ExamQuestionService questionService,ExamSubjectService subjectService,ScoreRateService scoreRateService,MarkLibraryService libraryService) {
        this.exam = exam;
        this.examService = examService;
        this.studentService = studentService;
        this.questionService = questionService;
        this.subjectService = subjectService;
        this.scoreRateService = scoreRateService;
        this.libraryService = libraryService;
        this.objectiveMap = new HashMap<String, List<ExamQuestion>>();
        this.subjectiveMap = new HashMap<String, List<ExamQuestion>>();
        this.scoreRateMap = new HashMap<String, Double>();
        this.campusNameMap = new HashMap<String, Set<String>>();
    }

    @Override
    public void run() {
        log.info("start calculate for examId=" + exam.getId());
        try {
            int pageNumber = 1;
            int pageSize = 1000;
            List<ExamStudent> list = studentService.findByExamIdAndUploadAndAbsent(exam.getId(), true, false,
                    pageNumber, pageSize);
            while (list != null && list.size() > 0) {
                for (ExamStudent student : list) {
                    calculate(student);
                }

                pageNumber++;
                list = studentService.findByExamIdAndUploadAndAbsent(exam.getId(), true, false, pageNumber, pageSize);
            }
            //统计各科目下每题得分率,主观题评卷完成才统计
            long total = subjectService.sumLibraryCount(exam.getId(), 0, 0);
            long finish = libraryService.countByExamAndStatus(exam.getId(), LibraryStatus.MARKED);
            int percent = total > 0 ? (int) (finish * 100.00 / total) : 0;
            if(percent>=100){
            	statistics();
            }
            
        } catch (Exception e) {
            log.error("calculate exception for examId=" + exam.getId(), e);
        } finally {
            exam.setStatus(ExamStatus.START);
            examService.save(exam);

            log.info("finish calculate for examId=" + exam.getId());
        }
    }

	private void statistics() {
		List<ExamSubject> examSubjects = subjectService.list(exam.getId());
		for (ExamSubject examSubject : examSubjects) {
			Set<String> campusNameList = campusNameMap.get(examSubject.getCode());
			for (String campusName : campusNameList) {
				List<ExamQuestion> questions = this.findQuestionList(examSubject.getCode(), false);
				Long count = studentService.countByExamIdAndSubjectCodeAndCampus(exam.getId(),examSubject.getCode(), campusName, true,false);
				for (ExamQuestion examQuestion : questions) {
					ScoreRate scoreRate = scoreRateService.findOne(exam.getId(), examSubject.getCode(), campusName,false, examQuestion.getMainNumber(),examQuestion.getSubNumber());
					if (scoreRate == null) {
						scoreRate = new ScoreRate();
						scoreRate.setExamId(exam.getId());
						scoreRate.setCampusName(campusName);
						scoreRate.setSubjectCode(examSubject.getCode());
						scoreRate.setMainNumber(examQuestion.getMainNumber());
						scoreRate.setSubNumber(examQuestion.getSubNumber());
						scoreRate.setObjective(examQuestion.isObjective());
						scoreRate.setMainTitle(examQuestion.getMainTitle());
						scoreRate.setTotalScore(examQuestion.getTotalScore());
					}
					String key = getKey(examSubject.getCode(), campusName,examQuestion.getQuestionNumber());
					BigDecimal bg = new BigDecimal(scoreRateMap.get(key) * 1.00 / count);
					double avgScore = bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
					if(scoreRate.getAvgScore()==null || scoreRate.getAvgScore()!=avgScore){
						scoreRate.setAvgScore(avgScore);
					}
					if(scoreRate.getTotalScore()!=examQuestion.getTotalScore()){
						scoreRate.setTotalScore(examQuestion.getTotalScore());
					}
					scoreRateService.save(scoreRate);
				}
			}
		}
	}

    private void calculate(ExamStudent student) {
        ScoreCalculateUtil util = ScoreCalculateUtil.instance(student);
        ScoreInfo info = util.calculate(findQuestionList(student.getSubjectCode(),true), null);

        student.setObjectiveScore(info.getObjectiveScore());
        student.setScoreList(info.getScoreList(), true);
        if(("".equals(student.getSubjectiveScoreString()) || "0".equals(student.getSubjectiveScoreString()))){//当考生的主观分等于0时，同时该考生的试卷已经评卷的情况，再获取一次评卷分数
        	MarkLibrary markLibrary = libraryService.findByStudentId(student.getId());
        	if(markLibrary != null){//并且存在评卷记录的
        		student.setSubjectiveScore(markLibrary.getMarkerScore());
        		student.setScoreList(markLibrary.getScoreList(), false);
        	}
        }
        studentService.save(student);
        
        if(!student.getScoreList(false).isEmpty()){//主观题有分数明细才统分
        	statistics(student);
        }
    }
    private void statistics(ExamStudent student) {
    	Set<String> campusNames = campusNameMap.get(student.getSubjectCode());
    	if(campusNames == null || campusNames.isEmpty()){
    		campusNames = new HashSet<>();
    		campusNames.add(student.getCampusName());
    	}else{
    		campusNames.add(student.getCampusName());
    	}
    	this.campusNameMap.put(student.getSubjectCode(), campusNames);
    	List<ScoreItem> items = student.getScoreList(false);
    	List<ExamQuestion> list = this.findQuestionList(student.getSubjectCode(),false);
    	if(!items.isEmpty() && list != null){
        	if(items.size()!=list.size()){
        		return ;
        	}
    	    for (int i = 0; i < list.size(); i++) {
    	        ScoreItem scoreItem = items.get(i);
    	        String key = getKey(student.getSubjectCode(),student.getCampusName(),list.get(i).getQuestionNumber());
    	        Double totalScore = scoreRateMap.get(key);
    	        if(totalScore==null){
    	            scoreRateMap.put(key,scoreItem.getScore());
    	        }else{
    	            scoreRateMap.put(key,totalScore+scoreItem.getScore());
    	        }
    	    }
    	}
	}

	private String getKey(String subjectCode,String campusName,String questionNumber) {
		List<String> list = new ArrayList<String>();
		list.add(subjectCode);
		list.add(campusName);
		list.add(questionNumber);
		return StringUtils.join(list, "-");
	}

	private List<ExamQuestion> findQuestionList(String subjectCode, boolean objective) {
    	if(objective){
    		List<ExamQuestion> list = objectiveMap.get(subjectCode);
    		if (list == null) {
    			list = questionService.findByExamAndSubjectAndObjective(exam.getId(), subjectCode, true);
    			objectiveMap.put(subjectCode, list);
    		}
    		return list;
    	}else{
    		List<ExamQuestion> list = subjectiveMap.get(subjectCode);
    		if (list == null) {
    			list = questionService.findByExamAndSubjectAndObjective(exam.getId(), subjectCode, false);
    			subjectiveMap.put(subjectCode, list);
    		}
    		return list;
    	}
    }
}
